import stan
import matplotlib.pyplot as plt
import numpy as np
import arviz as az

OneVar_code = """
data {
        int<lower=0> J;
        array[J] real y;
}
parameters {
        real vmu;
        real<lower=0> vsigma;
}
transformed parameters {
        //        Rescaling of vmu and vsigma
        real mu;
        real sigma;
        mu = vmu * 1000.0;
        sigma = vsigma * 1000.0 + 0.001;
}
model {
        vmu ~ normal(0.0, 1.0);
        vsigma ~ exponential(1.0);
        y ~ normal(mu, sigma);
}
"""

LData =  [ 44, 44, 56, 59, 44, 44, 63, 60, 34, 58,
           55, 50, 32, 61, 47, 25, 38, 47, 30, 42]

OneVar_dat = { 'J' : len(LData), 'y': LData }

sm = stan.build(OneVar_code, data=OneVar_dat)
fit = sm.sample()
#
#     InferenceData class
#
infdata = az.from_pystan(posterior = fit, posterior_model = sm)

summary = az.summary(infdata, var_names=['mu', 'sigma'])
print('\nSummary...\n', summary)

az.plot_trace(infdata, var_names=['mu', 'sigma'])
plt.tight_layout()
plt.show()
#
#      Pandas DataFrame
#
dframe = fit.to_frame()
#
#      Calculation of median based-summaries,
#      which are recommended by Gelman et al.(2021)
#
mu_med = np.median(dframe['mu'])
mu_mad_sd = 1.483 * np.median(np.abs(dframe['mu'] - mu_med))
print(f'\nmu_med = {mu_med:.3f},   mu_mad_sd = {mu_mad_sd:.3f}')
sigma_med = np.median(dframe['sigma'])
sigma_mad_sd = 1.483 * np.median(np.abs(dframe['sigma'] - sigma_med))
print(f'sigma_med = {sigma_med:.3f},   sigma_mad_sd = {sigma_mad_sd:.3f}')
