import numpy as np
import stan
import matplotlib.pyplot as plt
import arviz as az

Data = [
 [179, 121], [185, 89], [181, 106], [210, 100], [202, 104],
 [197, 91], [227, 107], [208, 101], [188, 99], [189, 112],
 [240, 111], [203, 103], [202, 85], [190, 95], [221, 96],
 [150, 83], [205, 103], [177, 90], [221, 92], [172, 86],
 [190, 87], [218, 101], [205, 100], [208, 120], [224, 98],
 [199, 116], [203, 96], [213, 91], [200, 109], [190, 100],
 [211, 115], [204, 85], [185, 92], [178, 86], [231, 122],
 [206, 84], [197, 93], [236, 93], [213, 106], [205, 103],
 [212, 102], [177, 86], [193, 96], [183, 93], [204, 87],
 [203, 108], [203, 103], [180, 100], [196, 101], [167, 75],
 [202, 106], [182, 98], [225, 109], [219, 115], [205, 102],
 [200, 99], [219, 94], [189, 102], [220, 86], [186, 102],
 [203, 101], [210, 124], [254, 103], [208, 95], [192, 106],
 [176, 91], [173, 97], [195, 100], [189, 98], [220, 115],
 [202, 112], [257, 108], [207, 104], [199, 96], [173, 98],
 [230, 101], [230, 111], [212, 105], [196, 82], [183, 82],
 [230, 103], [207, 112], [225, 101], [192, 104], [175, 83],
 [223, 105], [192, 106], [202, 87], [204, 87], [184, 97],
 [193, 94], [204, 109], [196, 92], [216, 109], [219, 113],
 [256, 117], [210, 86], [217, 103], [206, 87], [213, 105] ]

mu = np.mean(Data, axis = 0)
print('mu =\n', mu)
cov = np.cov(np.array(Data).T)
print('cov =\n', cov)
sgms = [cov[0][0]**0.5, cov[1][1]**0.5]
print('sgm =\n', sgms) 

L1 = mu[0] - 5 * sgms[0]
U1 = mu[0] + 5 * sgms[0]
L2 = mu[1] - 5 * sgms[1]
U2 = mu[1] + 5 * sgms[1]
sgm_L1 = 0.0
sgm_U1 = 10 * sgms[0]
sgm_L2 = 0.0
sgm_U2 = 10 * sgms[1]

stan_data = {'N': len(Data), 'Data': Data, 'l1': L1, 'u1': U1, 'l2': L2, 'u2': U2,
			'sgm_l1': sgm_L1, 'sgm_u1': sgm_U1, 
			'sgm_l2': sgm_L2, 'sgm_u2': sgm_U2}

with open('TwoVarNormal.stan', 'r') as f_stan:
        sm = stan.build(f_stan.read(), data=stan_data)
fit = sm.sample()
#
#           InferenceData class
#
infdata = az.from_pystan(posterior = fit, posterior_model = sm)
summary = az.summary(infdata,var_names=['mu1', 'mu2',  'sgm1', 'sgm2', 'r'])
print('\nSummary...\n', summary)

az.plot_trace(infdata, var_names=['mu1', 'mu2',  'sgm1', 'sgm2', 'r'],
              figsize=(10,6))
plt.tight_layout()
plt.show()
#
#      Pandas DataFrame
#
dframe = fit.to_frame()
#
#      Calculation of median based-summaries,
#      which are recommended by Gelman et al.(2021)
mu1_med = np.median(dframe['mu1'])
mu1_mad_sd = 1.483 * np.median(np.abs(dframe['mu1'] - mu1_med))
print(f'\nmu1_med = {mu1_med:.3f},   mu1_mad_sd = {mu1_mad_sd:.3f}')
mu2_med = np.median(dframe['mu2'])
mu2_mad_sd = 1.483 * np.median(np.abs(dframe['mu2'] - mu2_med))
print(f'mu2_med = {mu2_med:.3f},   mu2_mad_sd = {mu2_mad_sd:.3f}')
sgm1_med = np.median(dframe['sgm1'])
sgm1_mad_sd = 1.483 * np.median(np.abs(dframe['sgm1'] - sgm1_med))
print(f'sgm1_med = {sgm1_med:.3f},   sgm1_mad_sd = {sgm1_mad_sd:.3f}')
sgm2_med = np.median(dframe['sgm2'])
sgm2_mad_sd = 1.483 * np.median(np.abs(dframe['sgm2'] - sgm2_med))
print(f'sgm2_med = {sgm2_med:.3f},   sgm2_mad_sd = {sgm2_mad_sd:.3f}')
r_med = np.median(dframe['r'])
r_mad_sd = 1.483 * np.median(np.abs(dframe['r'] - r_med))
print(f'r_med = {r_med:.3f},   r_mad_sd = {r_mad_sd:.3f}')

